package me.unurled.sacredrealms.sr.components.treasure;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import java.util.ArrayList;
import java.util.List;
import javax.annotation.Nullable;
import me.unurled.sacredrealms.sr.data.DataHandler;
import me.unurled.sacredrealms.sr.data.DataManager;
import me.unurled.sacredrealms.sr.managers.Manager;
import org.bukkit.Location;
import org.jetbrains.annotations.NotNull;

public class TreasureManager extends Manager {
  private final List<Treasure> treasures = new ArrayList<>();

  /** Save the data */
  @Override
  public void saveData() {
    // save all treasures
    for (Treasure treasure : treasures) {
      // serialize treasure
      DataHandler dh = DataManager.getInstance(DataManager.class).getDataHandler();
      Gson gson =
          new GsonBuilder().registerTypeAdapter(Treasure.class, new TreasureSerializer()).create();
      dh.set("treasures." + treasures.indexOf(treasure), gson.toJson(treasure));
    }
  }

  /** Load the data */
  @Override
  public void loadData() {
    // load all treasures
    DataHandler dh = DataManager.getInstance(DataManager.class).getDataHandler();
    Gson gson =
        new GsonBuilder().registerTypeAdapter(Treasure.class, new TreasureDeserializer()).create();
    for (String key : dh.getKeysAll("treasures")) {
      // deserialize treasure
      treasures.add(gson.fromJson(dh.get("treasures." + key), Treasure.class));
    }
  }

  public void addTreasure(Treasure treasure) {
    treasures.add(treasure);
  }

  @Nullable
  public Treasure findByLocation(@NotNull Location loc) {
    for (Treasure treasure : treasures) {
      if (treasure.getLocation().equals(loc)) {
        return treasure;
      }
    }
    return null;
  }

  @NotNull
  public List<Treasure> getTreasures() {
    return treasures;
  }

  public void removeTreasure(@NotNull Treasure treasure) {
    treasures.remove(treasure);
  }
}
