package me.unurled.sacredrealms.sr.components.treasure;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import java.lang.reflect.Type;
import me.unurled.sacredrealms.sr.components.item.ItemStackSerializer;
import org.bukkit.inventory.ItemStack;

public class TreasureSerializer implements JsonSerializer<Treasure> {

  /**
   * Gson invokes this call-back method during serialization when it encounters a field of the
   * specified type.
   *
   * <p>In the implementation of this call-back method, you should consider invoking {@link
   * JsonSerializationContext#serialize(Object, Type)} method to create JsonElements for any
   * non-trivial field of the {@code src} object. However, you should never invoke it on the {@code
   * src} object itself since that will cause an infinite loop (Gson will call your call-back method
   * again).
   *
   * @param src the object that needs to be converted to Json.
   * @param typeOfSrc the actual type (fully genericized version) of the source object.
   * @param context
   * @return a JsonElement corresponding to the specified object.
   */
  @Override
  public JsonElement serialize(Treasure src, Type typeOfSrc, JsonSerializationContext context) {
    JsonObject obj = new JsonObject();
    obj.add("location", new JsonPrimitive(src.getLocation().toString()));
    obj.add("permission", new JsonPrimitive(src.getPermission()));
    Gson gson =
        new GsonBuilder().registerTypeAdapter(ItemStack.class, new ItemStackSerializer()).create();
    obj.add("items", new JsonPrimitive(gson.toJson(src.getItems())));
    return obj;
  }
}
